==> ecuacion-splib-web-1.73.15-sources.jar: jp/ecuacion/splib/web/controller/SplibSearchListController.java <==
package jp.ecuacion.splib.web.controller;

import jakarta.annotation.Nonnull;
import java.util.Map;
import jp.ecuacion.lib.core.exception.checked.AppException;
import jp.ecuacion.lib.core.exception.checked.MultipleAppException;
import jp.ecuacion.splib.core.form.record.SplibRecord;
import jp.ecuacion.splib.web.advice.SplibExceptionHandler;
import jp.ecuacion.splib.web.advice.SplibExceptionHandlerData;
import jp.ecuacion.splib.web.enums.LoginStateEnum;
import jp.ecuacion.splib.web.exception.InputValidationException;
import jp.ecuacion.splib.web.form.SplibGeneralForm;
import jp.ecuacion.splib.web.form.SplibListForm;
import jp.ecuacion.splib.web.form.SplibSearchForm;
import jp.ecuacion.splib.web.service.SplibSearchListService;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;

public abstract class SplibSearchListController
    <S extends SplibSearchForm, L extends SplibListForm<? extends SplibRecord>>
    extends SplibGeneralController {

  protected abstract S getNewSearchForm();

  public abstract SplibSearchListService getService();

  public SplibSearchListController(LoginStateEnum loginState, @Nonnull String functionName,
      String rootRecordName) {
    super(loginState, functionName, rootRecordName);
  }

  public SplibSearchListController(LoginStateEnum loginState, @Nonnull String functionName) {
    super(loginState, functionName, functionName);
  }

  @GetMapping(value = "search")
  public String search(Model model, S f) throws Exception {
    S searchForm = getProperSearchForm(model, f);

    SplibListForm<? extends SplibRecord> listForm = getService().getListForm(searchForm);
    listForm.setMenuName(searchForm.getMenuName());
    model.addAttribute(functionName + "ListForm", listForm);

    return functionName + "SearchList";
  }

  /** list画面での削除時のエラー発生など、エラーがあって一覧画面に戻る場合に使用。 */
  public String search(Model model, String menuName) throws Exception {
    S form = getNewSearchForm();
    form.setMenuName(menuName);

    return search(model, form);
  }

  @SuppressWarnings({"unchecked"})
  protected S getProperSearchForm(Model model, S f) {

    String formName = functionName + "SearchForm";
    String key = getSessionKey(formName, f);
    if (f != null) {
      // 検索画面上で検索された場合はその条件を採用し保管。
      // それ以外の場合（メニューのリンクなどから来た場合など）はmodelに設定されたものをそのまま使用。（ここでは何もしない）
      if (f.isRequestFromSearchForm()) {
        request.getSession().setAttribute(key, f);
      }

      // 初回アクセスでsessionに存在しない場合は引数のformを使用
      if (request.getSession().getAttribute(key) == null) {
        request.getSession().setAttribute(key, f);
      }

    } else {
      // その前に検索しているはずなので、引数のfがnullで、かつsessionにも情報がない、という場合はあり得ない。
      if (request.getSession().getAttribute(formName) == null) {
        throw new RuntimeException("f == null cannot be occurred.");
      }
    }

    S formUsedForSearch = (S) request.getSession().getAttribute(getSessionKey(formName, f));

    // 使用するsearchFormがspring mvcからも使用されるよう、modelにも入れておく
    model.addAttribute(formName, formUsedForSearch);

    return formUsedForSearch;
  }

  /** fがnullの場合null Pointerが発生するためメソッド冒頭で定義することはできず、別メソッドとした。 */
  private String getSessionKey(String formName, S f) {
    return formName + (f == null || f.getMenuName() == null || f.getMenuName().equals("") ? ""
        : "." + f.getMenuName());
  }

  @GetMapping(value = "search", params = "conditionClear")
  public String searchConditionClear(Model model, S f) throws MultipleAppException {
    // 情報をクリアするための設定を行う
    String formName = functionName + "SearchForm";
    String sessionKey = formName
        + (f.getMenuName() == null || f.getMenuName().equals("") ? "" : "." + f.getMenuName());
    request.getSession().setAttribute(sessionKey, null);

    return "redirect:/" + loginState.getCode() + "/" + functionName + "/search?"
        + getUrlParams(f.getMenuName(), false);
  }

  /**
   * 一覧からの削除処理。 delete用URLをお気に入りに入れられても困るので、本当はPostでの通信にしたいところだったが、
   * 呼び出し先を、showUpdateForm()と同じ「action」にしており、その状態でdeleteの場合のみpostに変更して投げてもspring mvcが判別つかないようで、
   * 「postは無効です」的なエラーが発生する。
   * deleteのみ別のformとするのもやりにくいし、formで持つactionをjavascriptで書き換えるのも微妙（th:actionでthymeleafが制御しているところなので）
   * なことから、GETとすることを許容する。以下の対策を行うことで、実質問題も起こらないと考えられる。 -
   * PRGを使うことで、delete中にシステムエラーでも起きない限り、delete時のURLがブラウザのURLバーには残らない
   */
  @GetMapping(value = "list", params = "delete")
  public String delete(Model model, L f, @AuthenticationPrincipal UserDetails loginUser)
      throws Exception {
    prepareWithMovingToListOnError(f, model);
    getService().delete(f, loginUser);

    return "redirect:/" + loginState.getCode() + "/" + functionName + "/search?success"
        + getUrlParams(f.getMenuName(), true);
  }

  protected String getUrlParams(String menuName, boolean needsAmpersand) {
    String additionalString = "";
    if (menuName != null && !menuName.equals("")) {
      additionalString = (needsAmpersand ? "&" : "") + "menuName=" + menuName;
    }

    return additionalString;
  }

  @GetMapping(value = "list", params = "showInsertForm")
  public String showInsertForm(Model model) {
    return "redirect:/" + loginState.getCode() + "/" + functionName + "/edit?showInsertForm=";
  }

  @GetMapping(value = "list", params = "showUpdateForm")
  public String showUpdateForm(Model model) {
    Map<String, String[]> paramMap = request.getParameterMap();
    StringBuilder sb = new StringBuilder();
    paramMap.keySet().stream().forEach(key -> sb.append("&" + key + "=" + paramMap.get(key)[0]));
    return "redirect:/" + loginState.getCode() + "/" + functionName + "/edit?showUpdateForm="
        + sb.toString();
  }

  @Override
  protected void prepare(SplibGeneralForm form, Model model, String nextPageOnError,
      BindingResult result) throws InputValidationException, AppException {

    super.prepare(form, model, nextPageOnError, result);
  }

  /**
   * 一覧で対象を指定し削除、など、list画面での処理の場合、formには削除対象のデータが入っているだけで一覧全体の情報は持っていない。
   * そのため画面を再表示するには、エラーメッセージを詰めたmodelを引き渡しつつ、list表示用の処理を呼び出し再度一覧情報を取得する流れとなる。
   * <p>
   * その場合、削除などの処理終了後にlist表示用の処理を呼び出すことになるのだが、reflectionを使用しそのメソッドを引数に渡してもらう、
   * というのも煩雑なので、一覧表示用のlistは「public String list(Model model)」と固定し、そこに戻る処理とする。
   * </p>
   */
  protected void prepareWithMovingToListOnError(SplibGeneralForm form, Model model,
      BindingResult result) throws InputValidationException, AppException {
    // 1. preparation
    SplibExceptionHandlerData info = new SplibExceptionHandlerData(this, form);
    request.setAttribute(SplibExceptionHandler.INFO_FOR_ERROR_HANDLING, info);

    commonProc(form, model, result, false);
  }

  /**
   * formチェックがない場合は、input validationのないこちらを使用。
   */
  protected void prepareWithMovingToListOnError(SplibGeneralForm form, Model model)
      throws InputValidationException, AppException {
    prepareWithMovingToListOnError(form, model, null);
  }
}

==> ecuacion-splib-web-1.74.1-sources.jar: jp/ecuacion/splib/web/controller/SplibSearchListController.java <==
package jp.ecuacion.splib.web.controller;

import jakarta.annotation.Nonnull;
import jp.ecuacion.lib.core.exception.checked.AppException;
import jp.ecuacion.lib.core.exception.checked.MultipleAppException;
import jp.ecuacion.splib.core.form.record.SplibRecord;
import jp.ecuacion.splib.web.advice.SplibExceptionHandler;
import jp.ecuacion.splib.web.advice.SplibExceptionHandlerData;
import jp.ecuacion.splib.web.bean.RedirectUrlOnSuccessBean;
import jp.ecuacion.splib.web.bean.RedirectUrlOnSuccessPageBean;
import jp.ecuacion.splib.web.enums.LoginStateEnum;
import jp.ecuacion.splib.web.exception.InputValidationException;
import jp.ecuacion.splib.web.form.SplibGeneralForm;
import jp.ecuacion.splib.web.form.SplibListForm;
import jp.ecuacion.splib.web.form.SplibSearchForm;
import jp.ecuacion.splib.web.service.SplibSearchListService;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;

public abstract class SplibSearchListController
    <S extends SplibSearchForm, L extends SplibListForm<? extends SplibRecord>>
    extends SplibGeneralController {

  protected abstract S getNewSearchForm();

  public abstract SplibSearchListService getService();

  public SplibSearchListController(LoginStateEnum loginState, @Nonnull String functionName,
      String rootRecordName) {
    super(loginState, functionName, rootRecordName);
  }

  public SplibSearchListController(LoginStateEnum loginState, @Nonnull String functionName) {
    super(loginState, functionName, functionName);
  }

  @GetMapping(value = "search")
  public String search(Model model, S f) throws Exception {
    S searchForm = getProperSearchForm(model, f);

    SplibListForm<? extends SplibRecord> listForm = getService().getListForm(searchForm);
    listForm.setMenuName(searchForm.getMenuName());
    model.addAttribute(functionName + "ListForm", listForm);

    return functionName + "SearchList";
  }

  /** list画面での削除時のエラー発生など、エラーがあって一覧画面に戻る場合に使用。 */
  public String search(Model model, String menuName) throws Exception {
    S form = getNewSearchForm();
    form.setMenuName(menuName);

    return search(model, form);
  }

  @SuppressWarnings({"unchecked"})
  protected S getProperSearchForm(Model model, S f) {

    String formName = functionName + "SearchForm";
    String key = getSessionKey(formName, f);
    if (f != null) {
      // 検索画面上で検索された場合はその条件を採用し保管。
      // それ以外の場合（メニューのリンクなどから来た場合など）はmodelに設定されたものをそのまま使用。（ここでは何もしない）
      if (f.isRequestFromSearchForm()) {
        request.getSession().setAttribute(key, f);
      }

      // 初回アクセスでsessionに存在しない場合は引数のformを使用
      if (request.getSession().getAttribute(key) == null) {
        request.getSession().setAttribute(key, f);
      }

    } else {
      // その前に検索しているはずなので、引数のfがnullで、かつsessionにも情報がない、という場合はあり得ない。
      if (request.getSession().getAttribute(formName) == null) {
        throw new RuntimeException("f == null cannot be occurred.");
      }
    }

    S formUsedForSearch = (S) request.getSession().getAttribute(getSessionKey(formName, f));

    // 使用するsearchFormがspring mvcからも使用されるよう、modelにも入れておく
    model.addAttribute(formName, formUsedForSearch);

    return formUsedForSearch;
  }

  /** fがnullの場合null Pointerが発生するためメソッド冒頭で定義することはできず、別メソッドとした。 */
  private String getSessionKey(String formName, S f) {
    return formName + (f == null || f.getMenuName() == null || f.getMenuName().equals("") ? ""
        : "." + f.getMenuName());
  }

  @GetMapping(value = "search", params = "conditionClear")
  public String searchConditionClear(Model model, S f) throws MultipleAppException {
    // 情報をクリアするための設定を行う
    String formName = functionName + "SearchForm";
    String sessionKey = formName
        + (f.getMenuName() == null || f.getMenuName().equals("") ? "" : "." + f.getMenuName());
    request.getSession().setAttribute(sessionKey, null);

    return "redirect:/" + loginState.getCode() + "/" + functionName + "/search?"
        + getUrlParams(f.getMenuName(), false);
  }

  /**
   * 一覧からの削除処理。 delete用URLをお気に入りに入れられても困るので、本当はPostでの通信にしたいところだったが、
   * 呼び出し先を、showUpdateForm()と同じ「action」にしており、その状態でdeleteの場合のみpostに変更して投げてもspring mvcが判別つかないようで、
   * 「postは無効です」的なエラーが発生する。
   * deleteのみ別のformとするのもやりにくいし、formで持つactionをjavascriptで書き換えるのも微妙（th:actionでthymeleafが制御しているところなので）
   * なことから、GETとすることを許容する。以下の対策を行うことで、実質問題も起こらないと考えられる。 -
   * PRGを使うことで、delete中にシステムエラーでも起きない限り、delete時のURLがブラウザのURLバーには残らない
   */
  @GetMapping(value = "list", params = "delete")
  public String delete(Model model, L f, @AuthenticationPrincipal UserDetails loginUser)
      throws Exception {
    prepareWithMovingToListOnError(f, model);
    getService().delete(f, loginUser);

    return "redirect:/" + loginState.getCode() + "/" + functionName + "/search?success"
        + getUrlParams(f.getMenuName(), true);
  }

  protected String getUrlParams(String menuName, boolean needsAmpersand) {
    String additionalString = "";
    if (menuName != null && !menuName.equals("")) {
      additionalString = (needsAmpersand ? "&" : "") + "menuName=" + menuName;
    }

    return additionalString;
  }

  @GetMapping(value = "list", params = "showInsertForm")
  public String showInsertForm(Model model) {
    RedirectUrlOnSuccessBean bean =
        new RedirectUrlOnSuccessPageBean("edit", request.getParameterMap());
    return getActionSuccessRedirectString(bean);
  }

  @GetMapping(value = "list", params = "showUpdateForm")
  public String showUpdateForm(Model model) {
    RedirectUrlOnSuccessBean bean =
        new RedirectUrlOnSuccessPageBean("edit", request.getParameterMap());
    return getActionSuccessRedirectString(bean);
  }

  @Override
  protected void prepare(SplibGeneralForm form, Model model, String nextPageOnError,
      BindingResult result) throws InputValidationException, AppException {

    super.prepare(form, model, nextPageOnError, result);
  }

  /**
   * 一覧で対象を指定し削除、など、list画面での処理の場合、formには削除対象のデータが入っているだけで一覧全体の情報は持っていない。
   * そのため画面を再表示するには、エラーメッセージを詰めたmodelを引き渡しつつ、list表示用の処理を呼び出し再度一覧情報を取得する流れとなる。
   * <p>
   * その場合、削除などの処理終了後にlist表示用の処理を呼び出すことになるのだが、reflectionを使用しそのメソッドを引数に渡してもらう、
   * というのも煩雑なので、一覧表示用のlistは「public String list(Model model)」と固定し、そこに戻る処理とする。
   * </p>
   */
  protected void prepareWithMovingToListOnError(SplibGeneralForm form, Model model,
      BindingResult result) throws InputValidationException, AppException {
    // 1. preparation
    SplibExceptionHandlerData info = new SplibExceptionHandlerData(this, form);
    request.setAttribute(SplibExceptionHandler.INFO_FOR_ERROR_HANDLING, info);

    commonProc(form, model, result, false);
  }

  /**
   * formチェックがない場合は、input validationのないこちらを使用。
   */
  protected void prepareWithMovingToListOnError(SplibGeneralForm form, Model model)
      throws InputValidationException, AppException {
    prepareWithMovingToListOnError(form, model, null);
  }
}
